'**************************************
'*  NTSC 256 x 192 - luma/color bars  *
'**************************************

' P0 = YIQ

CON

  _clkmode = xinput
  _xinfreq = 60_000_000

  s		= 46		'scales DAC output (s = 0..128)
  r		= s * 78 / 128	'adjusts for modulator expansion

  lntsc         = 3640          'NTSC colorburst cycles per line * 16
  sntsc         = 624           'NTSC colorburst cycles per sync * 16


DAT		org

		reps	#64,#2			'start 8-bit incrementing sequence at $BB
		setspa	#$BB
		pusha	luma
		add	luma,lumad

		setctra	#%00111			'set ctra for PLL*8
                setfrqa	frqa_			'set frqa

		setvid	#%1110			'set vid for composite output
		setvidy	vidy_			'set yiq coefficients
		setvidi	vidi_
		setvidq	vidq_

		cfgdac0	#%11			'set bottom dac to video mode (YIQ)

		cfgpins	pinmask,pindacs		'set P0 for dac output
'
'
' Field loop
'
field           mov	x,#34			'top blanks
		call	#blank

                mov     x,#192			'set visible lines
line	        call	#hsync			'do horizontal sync
		waitvid	v_bo,#0			'black left border
		waitvid	mode,#0			'256*9 clocks per waitvid - show luma/colorbars
		waitvid	v_bo,#0			'black right border
		djnz    x,#line           	'another line?

                mov     x,#28			'bottom blanks
		call	#blank

		mov	x,#6			'high vertical syncs
		call	#vsynch

		mov	x,#6			'low vertical syncs
		call	#vsyncl

		mov	x,#6			'high vertical syncs
		call	#vsynch

                jmp     #field                  'field loop



blank		call	#hsync			'blank lines
		waitvid	v_bl,#0
		djnz	x,#blank
blank_ret	ret


hsync		waitvid	v_bs,c_sh		'horizontal sync
		waitvid	v_sn,c_sl
hsync_ret	retd
		waitvid	v_bc,c_sh
		waitvid	v_cb,c_cb
		waitvid	v_bv,c_sh


vsynch		waitvid	v_bs,c_sh		'vertical sync high
		waitvid	v_hs,c_sl
		waitvid	v_hr,c_sh
		djnz	x,#vsynch
vsynch_ret	ret


vsyncl		waitvid	v_bs,c_sh		'vertical sync low
		waitvid	v_ls,c_sl
		waitvid	v_lr,c_sh
		djnz	x,#vsyncl
vsyncl_ret	ret
'
'
' Initialized data
'
luma		long	$03020100
lumad		long	$04040404

frqa_		long	$1E8B_A2A7		'57.272720 MHz (colorburst * 16)

vidy_		long	(110*s/128 & $FF) << 24 + ((+38*s/128) & $FF) << 16 + ((+75*s/128) & $FF) << 8 + ((+15*s/128) & $FF)
vidi_		long	(100*s/128 & $FF) << 24 + ((+76*r/128) & $FF) << 16 + ((-35*r/128) & $FF) << 8 + ((-41*r/128) & $FF)
vidq_		long	              128 << 24 + ((+27*r/128) & $FF) << 16 + ((-67*r/128) & $FF) << 8 + ((+40*r/128) & $FF)

pinmask		long	%1			'pin mask for P0
pindacs		long	%1111_101100000		'pin configuration for video DAC

		'	color base	mode		clocks/pixel	clocks/waitvid
mode		long	$BB << 24 +	$A << 20 +	9 << 13 +	256*9

v_bo		long	(lntsc-sntsc-256*9)/2	'left/right border

v_bs		long	100			'before sync		horizontal sync
v_sn		long	269			'sync
v_bc		long	50			'before colorburst
v_cb		long	144			'colorburst
v_bv		long	sntsc-144-50-269-100	'before visible

v_hs		long	135			'high sync		high vertical sync
v_hr		long	lntsc/2-135-100		'high sync remainder

v_ls		long	1564			'low sync		low vertical sync
v_lr		long	lntsc/2-1564-100	'low sync remainder

v_bl		long	lntsc-sntsc		'blank visible

c_sh		long	$1_000000		'sync high
c_sl		long	$2_000000		'sync low
c_cb		long	$1_707000		'colorburst reference color

x		long	0
